@@ -5,10 +5,10 @@ package Stable::Module;
 #
 # http://search.cpan.org/dist/Stable-Module/
 #
-# Copyright (c) 2014, 2016, 2017, 2018, 2019 INABA Hitoshi <ina@cpan.org> in a CPAN
+# Copyright (c) 2014, 2016, 2017, 2018, 2019, 2023 INABA Hitoshi <ina@cpan.org> in a CPAN
 ######################################################################
 
-$VERSION = '0.09';
+$VERSION = '0.10';
 $VERSION = $VERSION;
 
 use 5.00503;
@@ -20,6 +20,10 @@ use IO::File;
 
 use vars qw($re_char $hide_stderr);
 
+# set OSNAME
+my $OSNAME = $^O;
+
+#---------------------------------------------------------------------
 sub VERSION {
     my($self,$version) = @_;
     if ($version != $Stable::Module::VERSION) {
@@ -28,6 +32,7 @@ sub VERSION {
     }
 }
 
+#---------------------------------------------------------------------
 sub BEGIN {
     if (($^O eq 'MSWin32') and (defined $ENV{'OS'}) and ($ENV{'OS'} eq 'Windows_NT')) {
         $hide_stderr = '2>NUL';
@@ -37,6 +42,7 @@ sub BEGIN {
     }
 }
 
+#---------------------------------------------------------------------
 sub import {
     my($caller,$filename,$line) = caller;
 
@@ -45,37 +51,20 @@ sub import {
         die "Incorrect use of module '${\__PACKAGE__}' at $filename line $line.\n";
     }
 
-    # get /./ (dot: one character) regexp
-    $re_char = qr/[\x00-\xFF]/;
-
-    # Code Page Identifiers (Microsoft Windows)
-    # Identifier .NET Name Additional information
+    # codepoint regexp
     my %re_char = (
-
-        # Sjis     shift_jis ANSI/OEM Japanese; Japanese (Shift-JIS)
-          '932' => qr{[\x81-\x9F\xE0-\xFC][\x00-\xFF]|[\x00-\xFF]},
-
-        # GBK      gb2312 ANSI/OEM Simplified Chinese (PRC, Singapore); Chinese Simplified (GB2312)
-          '936' => qr{[\x81-\xFE][\x00-\xFF]|[\x00-\xFF]},
-
-        # UHC      ks_c_5601-1987 ANSI/OEM Korean (Unified Hangul Code)
-          '949' => qr{[\x81-\xFE][\x00-\xFF]|[\x00-\xFF]},
-
-        # Big5Plus big5 ANSI/OEM Traditional Chinese (Taiwan; Hong Kong SAR, PRC); Chinese Traditional (Big5)
-          '950' => qr{[\x81-\xFE][\x00-\xFF]|[\x00-\xFF]},
-
-        # Big5HKSCS HKSCS support on top of traditional Chinese Windows
-          '951' => qr{[\x81-\xFE][\x00-\xFF]|[\x00-\xFF]},
-
-        # GB18030  GB18030 Windows XP and later: GB18030 Simplified Chinese (4 byte); Chinese Simplified (GB18030)
-        '54936' => qr{[\x81-\xFE][\x30-\x39][\x81-\xFE][\x30-\x39]|[\x81-\xFE][\x00-\xFF]|[\x00-\xFF]},
+        'sjis'      => qr{[\x81-\x9F\xE0-\xFC][\x00-\xFF]|[\x00-\xFF]},
+        'gbk'       => qr{[\x81-\xFE][\x00-\xFF]|[\x00-\xFF]},
+        'uhc'       => qr{[\x81-\xFE][\x00-\xFF]|[\x00-\xFF]},
+        'big5'      => qr{[\x81-\xFE][\x00-\xFF]|[\x00-\xFF]},
+        'big5hkscs' => qr{[\x81-\xFE][\x00-\xFF]|[\x00-\xFF]},
+        'eucjp'     => qr{[\xA1-\xFE][\x00-\xFF]|[\x00-\xFF]},
+        'gb18030'   => qr{[\x81-\xFE][\x30-\x39][\x81-\xFE][\x30-\x39]|[\x81-\xFE][\x00-\xFF]|[\x00-\xFF]},
+        'utf8'      => qr{[\xC2-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF][\x80-\xBF]|[\xF0-\xF4][\x80-\xBF][\x80-\xBF][\x80-\xBF]|[\x00-\xFF]},
     );
 
-    my $codepage = 'File System Safe';
-    if ($^O =~ m{\A (?:MSWin32|NetWare|symbian|dos) \z}oxms) {
-        $codepage = (qx{chcp} =~ m{ ([0-9]{3,5}) \Z}oxms)[0];
-        $re_char = $re_char{$codepage} || $re_char;
-    }
+    # get /./ (dot: one character) regexp
+    $re_char = ($re_char{detect_system_encoding()} || qr/[\x00-\xFF]/);
 
     no strict 'refs';
 
@@ -97,7 +86,7 @@ sub import {
 
     # use File::Basename qw(fileparse basename dirname);
     # use internal routines always for compatibility -- for example, perl 5.6 has incompatible File::Basename
-    if (1 or exists $re_char{$codepage}) {
+    if (defined $re_char) {
         $INC{'File/Basename.pm'} = join '/', $FindBin::Bin, __FILE__;
         *{$caller.'::fileparse'} = \&_fileparse;
         *{$caller.'::basename'}  = \&_basename;
@@ -111,7 +100,7 @@ sub import {
     }
 
     # use File::Path qw(mkpath rmtree);
-    if (exists $re_char{$codepage}) {
+    if (defined $re_char) {
         $INC{'File/Path.pm'}  = join '/', $FindBin::Bin, __FILE__;
         *{$caller.'::mkpath'} = \&_mkpath;
         *{$caller.'::rmtree'} = \&_rmtree;
@@ -123,7 +112,7 @@ sub import {
     }
 
     # use File::Copy qw(copy move);
-    if (exists $re_char{$codepage}) {
+    if (defined $re_char) {
         $INC{'File/Copy.pm'} = join '/', $FindBin::Bin, __FILE__;
         *{$caller.'::copy'}  = \&_copy;
         *{$caller.'::move'}  = \&_move;
@@ -177,10 +166,137 @@ sub import {
     *IO::Handle::say   = \&_say if not defined(*IO::Handle::say);
 }
 
+#---------------------------------------------------------------------
 sub unimport {
     # nothing
 }
 
+#---------------------------------------------------------------------
+# detect system encoding any of big5, big5hkscs, eucjp, gb18030, gbk, rfc2279, sjis, uhc, utf8, wtf8
+sub detect_system_encoding {
+
+    # running on Microsoft Windows
+    if ($OSNAME =~ /MSWin32/) {
+        if (my($system_encoding) = qx{chcp} =~ m/[^0123456789](932|936|949|950|951|20932|54936)\Z/) {
+            return {qw(
+                932   sjis
+                936   gbk
+                949   uhc
+                950   big5
+                951   big5hkscs
+                20932 eucjp
+                54936 gb18030
+            )}->{$system_encoding};
+        }
+        else {
+            return 'utf8';
+        }
+    }
+
+    # running on Oracle Solaris
+    elsif ($OSNAME =~ /solaris/) {
+        my $LANG =
+            defined($ENV{'LC_ALL'})   ? $ENV{'LC_ALL'}   :
+            defined($ENV{'LC_CTYPE'}) ? $ENV{'LC_CTYPE'} :
+            defined($ENV{'LANG'})     ? $ENV{'LANG'}     :
+            '';
+        return {qw(
+            ja_JP.PCK     sjis
+            ja            eucjp
+            japanese      eucjp
+            ja_JP.eucJP   eucjp
+            zh            gbk
+            zh.GBK        gbk
+            zh_CN.GBK     gbk
+            zh_CN.EUC     gbk
+            zh_CN.GB18030 gb18030
+            ko            uhc
+            ko_KR.EUC     uhc
+            zh_TW.BIG5    big5
+            zh_HK.BIG5HK  big5hkscs
+        )}->{$LANG} || 'utf8';
+    }
+
+    # running on HP HP-UX
+    elsif ($OSNAME =~ /hpux/) {
+        my $LANG =
+            defined($ENV{'LC_ALL'})   ? $ENV{'LC_ALL'}   :
+            defined($ENV{'LC_CTYPE'}) ? $ENV{'LC_CTYPE'} :
+            defined($ENV{'LANG'})     ? $ENV{'LANG'}     :
+            '';
+        return {qw(
+            japanese      sjis
+            ja_JP.SJIS    sjis
+            japanese.euc  eucjp
+            ja_JP.eucJP   eucjp
+            zh_CN.hp15CN  gbk
+            zh_CN.gb18030 gb18030
+            ko_KR.eucKR   uhc
+            zh_TW.big5    big5
+            zh_HK.big5    big5hkscs
+            zh_HK.hkbig5  big5hkscs
+        )}->{$LANG} || 'utf8';
+    }
+
+    # running on IBM AIX
+    elsif ($OSNAME =~ /aix/) {
+        my $LANG =
+            defined($ENV{'LC_ALL'})   ? $ENV{'LC_ALL'}   :
+            defined($ENV{'LC_CTYPE'}) ? $ENV{'LC_CTYPE'} :
+            defined($ENV{'LANG'})     ? $ENV{'LANG'}     :
+            '';
+        return {qw(
+            Ja_JP            sjis
+            Ja_JP.IBM-943    sjis
+            ja_JP            eucjp
+            ja_JP.IBM-eucJP  eucjp
+            zh_CN            gbk
+            zh_CN.IBM-eucCN  gbk
+            Zh_CN            gb18030
+            Zh_CN.GB18030    gb18030
+            ko_KR            uhc
+            ko_KR.IBM-eucKR  uhc
+            Zh_TW            big5
+            Zh_TW.big-5      big5
+            Zh_HK            big5hkscs
+            Zh_HK.BIG5-HKSCS big5hkscs
+        )}->{$LANG} || 'utf8';
+    }
+
+    # running on Other Systems
+    else {
+        my $LANG =
+            defined($ENV{'LC_ALL'})   ? $ENV{'LC_ALL'}   :
+            defined($ENV{'LC_CTYPE'}) ? $ENV{'LC_CTYPE'} :
+            defined($ENV{'LANG'})     ? $ENV{'LANG'}     :
+            '';
+        return {qw(
+            japanese      sjis
+            ja_JP.SJIS    sjis
+            ja_JP.mscode  sjis
+            ja            eucjp
+            japan         eucjp
+            japanese.euc  eucjp
+            Japanese-EUC  eucjp
+            ja_JP         eucjp
+            ja_JP.ujis    eucjp
+            ja_JP.eucJP   eucjp
+            ja_JP.AJEC    eucjp
+            ja_JP.EUC     eucjp
+            Jp_JP         eucjp
+            zh_CN.EUC     gbk
+            zh_CN.GB2312  gbk
+            zh_CN.hp15CN  gbk
+            zh_CN.gb18030 gb18030
+            ko_KR.eucKR   uhc
+            zh_TW.Big5    big5
+            zh_TW.big5    big5
+            zh_HK.big5    big5hkscs
+        )}->{$LANG} || 'utf8';
+    }
+}
+
+#---------------------------------------------------------------------
 sub _fileparse {
     my($fullname,@suffixlist) = @_;
 
@@ -238,6 +354,7 @@ sub _fileparse {
     }
 }
 
+#---------------------------------------------------------------------
 sub _basename {
     my($fullname,@suffixlist) = @_;
 
@@ -305,6 +422,7 @@ sub _basename {
     return $name;
 }
 
+#---------------------------------------------------------------------
 sub _dirname {
     my($fullname) = @_;
 
@@ -326,6 +444,7 @@ sub _dirname {
     return $dirname;
 }
 
+#---------------------------------------------------------------------
 sub _strip_trailing_sep {
     my($dirname) = @_;
 
@@ -339,6 +458,7 @@ sub _strip_trailing_sep {
     return join '', @char;
 }
 
+#---------------------------------------------------------------------
 sub _mkpath {
     my $path = _dos_path($_[0]);
 
@@ -388,6 +508,7 @@ sub _mkpath {
     }
 }
 
+#---------------------------------------------------------------------
 sub _rmtree {
     my $root = _dos_path($_[0]);
 
@@ -427,6 +548,7 @@ sub _rmtree {
     }
 }
 
+#---------------------------------------------------------------------
 sub _copy {
     my $source = _dos_path($_[0]);
     my $dest   = _dos_path($_[1]);
@@ -442,6 +564,7 @@ sub _copy {
     }
 }
 
+#---------------------------------------------------------------------
 sub _move {
     my $source = _dos_path($_[0]);
     my $dest   = _dos_path($_[1]);
@@ -457,6 +580,7 @@ sub _move {
     }
 }
 
+#---------------------------------------------------------------------
 sub _dos_path {
     my($path) = @_;
 
@@ -466,6 +590,7 @@ sub _dos_path {
     return $path;
 }
 
+#---------------------------------------------------------------------
 sub _is_directory {
     my($unknown) = @_;
 
@@ -478,6 +603,7 @@ sub _is_directory {
     return undef;
 }
 
+#---------------------------------------------------------------------
 sub _MSWin32_5Cended_path {
     if ((@_ >= 1) and ($_[0] ne '')) {
         if ($^O =~ /\A (?: MSWin32 | NetWare | symbian | dos ) \z/oxms) {
@@ -490,6 +616,7 @@ sub _MSWin32_5Cended_path {
     return undef;
 }
 
+#---------------------------------------------------------------------
 sub _first(&@) {
     my $coderef = shift @_;
     for (@_) {
@@ -500,6 +627,7 @@ sub _first(&@) {
     return undef;
 }
 
+#---------------------------------------------------------------------
 sub _shuffle(@) {
     my @a = \(@_);
     my $n;
@@ -507,11 +635,13 @@ sub _shuffle(@) {
     return map { $n = rand($i--); (${$a[$n]}, $a[$n] = $a[$i])[0]; } @_;
 }
 
+#---------------------------------------------------------------------
 sub _uniq {
     my %seen = ();
     return grep { not $seen{$_}++ } @_;
 }
 
+#---------------------------------------------------------------------
 sub _say {
     my $currfh = select();
     my $handle;
@@ -540,8 +670,7 @@ Stable::Module - frequently used modules on Perl5 application
 
 =head1 DESCRIPTION
 
-Stable::Module provides frequently used modules on Perl5 application, on both
-modern Perl and traditional Perl.
+Stable::Module provides frequently used modules on Perl5 application, on both modern Perl and traditional Perl.
 
 Stable::Module works as:
 
@@ -557,12 +686,12 @@ Stable::Module works as:
   use List::MoreUtils qw(all any none notall uniq);
   use feature         qw(say);
 
-fileparse, basename, dirname, mkpath, rmtree, copy, and move can treat multibyte
-encoding of path name.
+fileparse, basename, dirname, mkpath, rmtree, copy, and move can treat multibyte encoding of path name.
 
 =head1 BUGS
 
-  It is not possible to specify a regular expression to @suffixes.
+It is not possible to specify a regular expression to @suffixes.
+
   my @fileparse = fileparse($path, @suffixes);
   my $basename  = basename($path, @suffixes);
 
@@ -574,20 +703,21 @@ This project was originated by INABA Hitoshi.
 
 =head1 LICENSE AND COPYRIGHT
 
-This software is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself. See L<perlartistic>.
+This software is free software; you can redistribute it and/or modify it under the same terms as Perl itself. See L<perlartistic>.
 
-This software is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 =head1 SEE ALSO
 
-=over 4
+=over 2
 
-=item * L<ina|http://search.cpan.org/~ina/> - CPAN
+=item *
 
-=item * L<The BackPAN|http://backpan.perl.org/authors/id/I/IN/INA/> - A Complete History of CPAN
+L<ina|http://search.cpan.org/~ina/> - CPAN
+
+=item *
+
+L<The BackPAN|http://backpan.perl.org/authors/id/I/IN/INA/> - A Complete History of CPAN
 
 =back
 
